/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package dirstateparser;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author Tim Boudreau
 */
public class FileEntry {
    private String name;
    private long offset;
    private final DirstateFile src;

    public FileEntry(long offset, DirstateFile src) {
        this.offset = offset;
        this.src = src;
    }

    public long offset() {
        return offset;
    }

    @Override
    public boolean equals(Object obj) {
        try {
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            final FileEntry other = (FileEntry) obj;
            String thisName = name();
            String oName = other.name();
            if ((thisName == null) ? (oName != null) : !thisName.equals(oName)) {
                return false;
            }
            return true;
        } catch (IOException ex) {
            Logger.getLogger(FileEntry.class.getName()).log(Level.SEVERE, null, ex);
            return false;
        }
    }

    @Override
    public int hashCode() {
        int hash = 7;
        try {
            name();
        } catch (IOException ex) {
            Logger.getLogger(FileEntry.class.getName()).log(Level.SEVERE, null, ex);
        }
        hash = 11 * hash + (this.name != null ? this.name.hashCode() : 0);
        hash = 11 * hash + (int) (this.offset ^ (this.offset >>> 32));
        return hash;
    }

    private ByteBuffer getData() throws IOException {
        ByteBuffer data = src.buf(offset);
        return data;
    }

    public byte state() throws IOException {
        return getData().get();
    }

    public int mode() throws IOException {
        ByteBuffer buf = getData();
        buf.position(1);
        return buf.asIntBuffer().get();
    }

    private int nameLength() throws IOException {
        ByteBuffer buf = getData();
        buf.position(13);
        return buf.asIntBuffer().get();
    }

    public long lastModified() throws IOException {
        ByteBuffer buf = getData();
        buf.position(9);
        return buf.asIntBuffer().get();
    }

    long length() throws IOException {
        return 17 + nameLength();
    }

    public synchronized String name() throws IOException {
        if (name == null) {
            ByteBuffer buf = getData();
            buf.position(17);
            int nameLen = nameLength();
            byte[] nameBytes = new byte[nameLen];
            buf.get(nameBytes);
            return new String (nameBytes, "US-ASCII"); //XXX UTF-8?
        }
        return name;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        try {
            sb.append (name());
            sb.append ("[mode: ");
            sb.append (mode());
            sb.append (" state: ");
            sb.append (state());
            sb.append (" location in dirstate file: ");
            sb.append (state());
            sb.append ("]");
        } catch (IOException e) {
            sb.append (e.getMessage());
        }
        return sb.toString();
    }
}
